@@ -24,13 +24,13 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 20801 $
+ * $Revision: 22096 $
  * $Author: rjongbloed $
- * $Date: 2008-09-01 03:05:12 +0000 (Mon, 01 Sep 2008) $
+ * $Date: 2009-02-21 03:25:23 +0000 (Sat, 21 Feb 2009) $
  */
 
-#ifndef __OPAL_CALL_H
-#define __OPAL_CALL_H
+#ifndef OPAL_OPAL_CALL_H
+#define OPAL_OPAL_CALL_H
 
 #ifdef P_USE_PRAGMA
 #pragma interface
@@ -164,7 +164,36 @@ class OpalCall : public PSafeObject
       OpalConnection & connection   ///<  Connection that indicates it is alerting
     );
 
-    /**Call back for alerting.
+    /**Call back for remote party is now responsible for completing the call.
+       This function is called when the remote system has been contacted and it
+       has accepted responsibility for completing, or failing, the call. This
+       is distinct from OnAlerting() in that it is not known at this time if
+       anything is ringing. This indication may be used to distinguish between
+       "transport" level error, in which case another host may be tried, and
+       that finalising the call has moved "upstream" and the local system has
+       no more to do but await a result.
+
+       If an application overrides this function, it should generally call the
+       ancestor version for correct operation.
+
+       The default behaviour does nothing.
+     */
+    virtual void OnProceeding(
+      OpalConnection & connection   ///<  Connection that is proceeeding
+    );
+
+    /**Call back for remote party being alerted.
+       This function is called after the connection is informed that the
+       remote endpoint is "ringing". Generally some time after the
+       MakeConnection() function was called, this is function is called.
+
+       If PFalse is returned the connection is aborted.
+
+       If an application overrides this function, it should generally call the
+       ancestor version for correct operation. An application would typically
+       only intercept this function if it wishes to do some form of logging.
+       For this you can obtain the name of the caller by using the function
+       OpalConnection::GetRemotePartyName().
 
        The default behaviour is to call SetAlerting() on all the other
        connections in the call.
@@ -448,10 +477,33 @@ class OpalCall : public PSafeObject
     const PTime & GetStartTime() const { return startTime; }
   //@}
 
-    virtual PBoolean StartRecording(const PFilePath & fn);
-    virtual bool IsRecording() const;
-    virtual void StopRecording();
-    void OnStopRecordAudio(const PString & callToken);
+
+    /**Start recording a call.
+       Current version saves to a WAV file. It may either mix the receive and
+       transmit audio stream to a single mono file, or the streams are placed
+       into the left and right channels of a stereo WAV file.
+      */
+    bool StartRecording(
+      const PFilePath & filename, ///< File into which to record
+      bool mono = false           ///< Record as mono/stereo
+    );
+
+    /**Indicate if recording is currently active on call.
+      */
+    bool IsRecording() const;
+
+    /** Stop a recording.
+        Returns true if the call does exists, an active call is not indicated.
+      */
+    void StopRecording();
+
+    /** Call back for having a frame of audio to record.
+      */
+    virtual void OnRecordAudio(const PString & streamId, const RTP_DataFrame & frame);
+
+    /** Call back on recording stopped.
+      */
+    virtual void OnStopRecordAudio(const PString & streamId);
 
   protected:
     void SetPartyNames();
@@ -486,7 +538,7 @@ class OpalCall : public PSafeObject
 };
 
 
-#endif // __OPAL_CALL_H
+#endif // OPAL_OPAL_CALL_H
 
 
 // End of File ///////////////////////////////////////////////////////////////
