@@ -24,9 +24,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 22538 $
+ * $Revision: 22902 $
  * $Author: rjongbloed $
- * $Date: 2009-05-07 03:43:27 +0000 (Thu, 07 May 2009) $
+ * $Date: 2009-06-16 04:58:09 +0000 (Tue, 16 Jun 2009) $
  */
 
 #ifndef OPAL_SIP_SDP_H
@@ -268,7 +268,9 @@ class SDPSessionDescription : public PObject
   PCLASSINFO(SDPSessionDescription, PObject);
   public:
     SDPSessionDescription(
-      const OpalTransportAddress & address = OpalTransportAddress()
+      time_t sessionId,
+      unsigned version,
+      const OpalTransportAddress & address
     );
 
     void PrintOn(ostream & strm) const;
@@ -296,8 +298,8 @@ class SDPSessionDescription : public PObject
       const OpalTransportAddress & address
     );
 	
-    PINDEX GetOwnerSessionId() const { return ownerSessionId; }
-    void SetOwnerSessionId(PINDEX value) { ownerSessionId = value; }
+    time_t GetOwnerSessionId() const { return ownerSessionId; }
+    void SetOwnerSessionId(time_t value) { ownerSessionId = value; }
 
     PINDEX GetOwnerVersion() const { return ownerVersion; }
     void SetOwnerVersion(PINDEX value) { ownerVersion = value; }
@@ -322,7 +324,7 @@ class SDPSessionDescription : public PObject
     PString sessionName;
 
     PString ownerUsername;
-    unsigned ownerSessionId;
+    time_t ownerSessionId;
     unsigned ownerVersion;
     OpalTransportAddress ownerAddress;
     OpalTransportAddress defaultConnectAddress;
