@@ -24,9 +24,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 21851 $
- * $Author: csoutheren $
- * $Date: 2008-12-22 01:35:12 +0000 (Mon, 22 Dec 2008) $
+ * $Revision: 23467 $
+ * $Author: rjongbloed $
+ * $Date: 2009-09-16 07:50:34 +0000 (Wed, 16 Sep 2009) $
  */
 
 #ifndef OPAL_SIP_SIPCON_H
@@ -101,6 +101,10 @@ class SIPConnection : public OpalRTPConnection
       */
     virtual PString GetPrefixName() const;
 
+    /**Get the protocol-specific unique identifier for this connection.
+     */
+    virtual PString GetIdentifier() const;
+
     /**Get the remote party address as URL.
        This will return the "best guess" at an address to use in a
        to call the user again later. Note that under some circumstances this may be
@@ -134,6 +138,36 @@ class SIPConnection : public OpalRTPConnection
       */
     virtual PString GetCalledPartyURL();
 
+    /**Get alerting type information of an incoming call.
+       The type of "distinctive ringing" for the call. The string is protocol
+       dependent, so the caller would need to be aware of the type of call
+       being made. Some protocols may ignore the field completely.
+
+       For SIP this corresponds to the string contained in the "Alert-Info"
+       header field of the INVITE. This is typically a URI for the ring file.
+
+       For H.323 this must be a string representation of an integer from 0 to 7
+       which will be contained in the Q.931 SIGNAL (0x34) Information Element.
+
+       Default behaviour returns an empty string.
+      */
+    virtual PString GetAlertingType() const;
+
+    /**Set alerting type information for outgoing call.
+       The type of "distinctive ringing" for the call. The string is protocol
+       dependent, so the caller would need to be aware of the type of call
+       being made. Some protocols may ignore the field completely.
+
+       For SIP this corresponds to the string contained in the "Alert-Info"
+       header field of the INVITE. This is typically a URI for the ring file.
+
+       For H.323 this must be a string representation of an integer from 0 to 7
+       which will be contained in the Q.931 SIGNAL (0x34) Information Element.
+
+       Default behaviour returns false.
+      */
+    virtual bool SetAlertingType(const PString & info);
+
     /**Initiate the transfer of an existing call (connection) to a new remote 
        party.
 
@@ -337,7 +371,10 @@ class SIPConnection : public OpalRTPConnection
 
     /**Handle an incoming Trying response PDU
       */
-    virtual void OnReceivedTrying(SIP_PDU & pdu);
+    virtual void OnReceivedTrying(
+      SIPTransaction & transaction,
+      SIP_PDU & response
+    );
   
     /**Handle an incoming Ringing response PDU
       */
@@ -369,7 +406,7 @@ class SIPConnection : public OpalRTPConnection
   
     /**Handle a sending INVITE request
       */
-    virtual void OnCreatingINVITE(SIP_PDU & pdu);
+    virtual void OnCreatingINVITE(SIPInvite & pdu);
 
     /**Send a "200 OK" response for the received INVITE message.
      */
@@ -410,17 +447,12 @@ class SIPConnection : public OpalRTPConnection
     virtual void OnReceivedMESSAGE(SIP_PDU & pdu);
     virtual void OnMessageReceived(const SIPURL & from, const SIP_PDU & pdu);
 
+    PString GetLocalPartyURL() const;
+
   protected:
     PDECLARE_NOTIFIER(PTimer, SIPConnection, OnInviteResponseRetry);
     PDECLARE_NOTIFIER(PTimer, SIPConnection, OnAckTimeout);
 
-    virtual RTP_UDP *OnUseRTPSession(
-      const unsigned rtpSessionId,
-      const OpalMediaType & mediaType,
-      const OpalTransportAddress & mediaAddress,
-      OpalTransportAddress & localAddress
-    );
-
     virtual bool OnSendSDP(
       bool isAnswerSDP,
       OpalRTPSessionManager & rtpSessions,
@@ -445,6 +477,18 @@ class SIPConnection : public OpalRTPConnection
       SDPSessionDescription & sdp,
       unsigned sessionId
     );
+
+    virtual OpalMediaSession * SetUpMediaSession(
+      const unsigned rtpSessionId,
+      const OpalMediaType & mediaType,
+      SDPMediaDescription * mediaDescription,
+      OpalTransportAddress & localAddress,
+      bool & remoteChanged
+    );
+
+    bool SendReINVITE(PTRACE_PARAM(const char * msg));
+    void StartPendingReINVITE();
+
     friend class SIPInvite;
     static PBoolean WriteINVITE(OpalTransport & transport, void * param);
     bool WriteINVITE(OpalTransport & transport);
@@ -458,7 +502,6 @@ class SIPConnection : public OpalRTPConnection
       SIPDialogNotification::Events eventType = SIPDialogNotification::NoEvent,
       unsigned eventCode = 0
     );
-    void ExtractAppearanceCode(SIP_PDU & pdu);
 
 
     // Member variables
@@ -480,11 +523,14 @@ class SIPConnection : public OpalRTPConnection
 
     SIP_PDU             * originalInvite;
     PTime                 originalInviteTime;
-
+    time_t                m_sdpSessionId;
+    unsigned              m_sdpVersion; // Really a sequence number
     bool                  needReINVITE;
+    bool                  m_handlingINVITE;
     SIPDialogContext      m_dialog;
-    PGloballyUniqueID     m_dialogNotifyId;
+    OpalGloballyUniqueID  m_dialogNotifyId;
     int                   m_appearanceCode;
+    PString               m_alertInfo;
     SIPAuthentication   * authentication;
 
     std::map<SIP_PDU::Methods, unsigned> m_lastRxCSeq;
@@ -495,6 +541,7 @@ class SIPConnection : public OpalRTPConnection
     bool                      ackReceived;
     PSafePtr<SIPTransaction>  referTransaction;
     PSafeList<SIPTransaction> forkedInvitations; // Not for re-INVITE
+    PSafeList<SIPTransaction> pendingInvitations; // For re-INVITE
 
     enum {
       ReleaseWithBYE,
@@ -505,12 +552,14 @@ class SIPConnection : public OpalRTPConnection
 
     OpalMediaFormatList remoteFormatList;
 
-    PString GetLocalPartyURL() const;
-
   protected:
     PTimer sessionTimer;
   public:
     PDECLARE_NOTIFIER(PTimer, SIPConnection, OnSessionTimeout);
+
+  private:
+    P_REMOVE_VIRTUAL_VOID(OnCreatingINVITE(SIP_PDU&));
+    P_REMOVE_VIRTUAL_VOID(OnReceivedTrying(SIP_PDU &));
 };
 
 
