@@ -24,9 +24,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 22994 $
+ * $Revision: 23284 $
  * $Author: rjongbloed $
- * $Date: 2009-06-26 06:07:27 +0000 (Fri, 26 Jun 2009) $
+ * $Date: 2009-08-28 00:16:56 +0000 (Fri, 28 Aug 2009) $
  */
 
 #ifndef OPAL_SIP_SIPEP_H
@@ -262,28 +262,6 @@ class SIPEndPoint : public OpalRTPEndPoint
       SIP_PDU * pdu
     );
 
-    /**Handle an incoming IntervalTooBrief response PDU
-      */
-    virtual void OnReceivedIntervalTooBrief(
-      SIPTransaction & transaction, 
-      SIP_PDU & response)
-    ;
-  
-    /**Handle an incoming Proxy Authentication Required response PDU
-      */
-    virtual void OnReceivedAuthenticationRequired(
-      SIPTransaction & transaction,
-      SIP_PDU & response
-    );
-
-    /**Handle an incoming OK response PDU.
-       This actually gets any PDU of the class 2xx not just 200.
-      */
-    virtual void OnReceivedOK(
-      SIPTransaction & transaction,
-      SIP_PDU & response
-    );
-    
     /**Handle an incoming NOTIFY PDU.
       */
     virtual PBoolean OnReceivedNOTIFY(
@@ -804,6 +782,7 @@ class SIPEndPoint : public OpalRTPEndPoint
 #endif
 
   protected:
+    void TransportReadLoop(OpalTransport * transport);
     PDECLARE_NOTIFIER(PThread, SIPEndPoint, TransportThreadMain);
     PDECLARE_NOTIFIER(PTimer, SIPEndPoint, NATBindingRefresh);
 
@@ -896,6 +875,12 @@ class SIPEndPoint : public OpalRTPEndPoint
 #if OPAL_HAS_SIPIM
     OpalSIPIMManager m_sipIMManager;
 #endif
+
+    bool m_disableTrying;
+
+    P_REMOVE_VIRTUAL_VOID(OnReceivedIntervalTooBrief(SIPTransaction &, SIP_PDU &));
+    P_REMOVE_VIRTUAL_VOID(OnReceivedAuthenticationRequired(SIPTransaction &, SIP_PDU &));
+    P_REMOVE_VIRTUAL_VOID(OnReceivedOK(SIPTransaction &, SIP_PDU &));
 };
 
 
