@@ -24,9 +24,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 22928 $
+ * $Revision: 23493 $
  * $Author: rjongbloed $
- * $Date: 2009-06-19 05:26:12 +0000 (Fri, 19 Jun 2009) $
+ * $Date: 2009-09-19 02:00:38 +0000 (Sat, 19 Sep 2009) $
  */
 
 #ifndef OPAL_SIP_SIPPDU_H
@@ -248,7 +248,7 @@ class SIPMIMEInfo : public PMIMEInfo
 
     void SetCompactForm(bool form) { compactForm = form; }
 
-    PCaselessString GetContentType() const;
+    PCaselessString GetContentType(bool includeParameters = false) const;
     void SetContentType(const PString & v);
 
     PCaselessString GetContentEncoding() const;
@@ -343,7 +343,7 @@ class SIPMIMEInfo : public PMIMEInfo
     PString GetEvent() const;
     void SetEvent(const PString & v);
     
-    PString GetSubscriptionState() const;
+    PCaselessString GetSubscriptionState() const;
     void SetSubscriptionState(const PString & v);
     
     PString GetUserAgent() const;
@@ -352,7 +352,7 @@ class SIPMIMEInfo : public PMIMEInfo
     PString GetOrganization() const;
     void SetOrganization(const PString & v);
 
-    void GetProductInfo(OpalProductInfo & info);
+    void GetProductInfo(OpalProductInfo & info) const;
     void SetProductInfo(const PString & ua, const OpalProductInfo & info);
 
     PString GetWWWAuthenticate() const;
@@ -551,6 +551,7 @@ class SIP_PDU : public PSafeObject
       IllegalStatusCode,
       Local_TransportError,
       Local_BadTransportAddress,
+      Local_Timeout,
 
       Information_Trying                  = 100,
       Information_Ringing                 = 180,
@@ -805,7 +806,7 @@ class SIPDialogContext
 
     void Update(const SIP_PDU & response);
 
-    unsigned GetNextCSeq() { return ++m_lastSentCSeq; }
+    unsigned GetNextCSeq(unsigned inc = 1) { return m_lastSentCSeq += inc; }
     bool IsDuplicateCSeq(unsigned sequenceNumber);
 
     bool IsEstablished() const { return !m_remoteTag.IsEmpty(); }
@@ -959,6 +960,21 @@ class SIPRegister : public SIPTransaction
 {
     PCLASSINFO(SIPRegister, SIPTransaction);
   public:
+    enum CompatibilityModes {
+      e_FullyCompliant,                 /**< Registrar is fully compliant, we will register
+                                             all listeners of all types (e.g. sip, sips etc)
+                                             in the Contact field. */
+      e_CannotRegisterMultipleContacts, /**< Registrar refuses to register more than one
+                                             contact field. Correct behaviour is to return
+                                             a contact with the fields it can accept in
+                                             the 200 OK */
+      e_CannotRegisterPrivateContacts   /**< Registrar refuses to register any RFC
+                                             contact field. Correct behaviour is to return
+                                             a contact with the fields it can accept in
+                                             the 200 OK */
+    };
+
+    /// Registrar parameters
     struct Params {
       Params();
 
@@ -972,7 +988,8 @@ class SIPRegister : public SIPTransaction
       unsigned      m_restoreTime;
       PTimeInterval m_minRetryTime;
       PTimeInterval m_maxRetryTime;
-      void          * m_userData;
+      void        * m_userData;
+      CompatibilityModes m_compatibility;
     };
 
     SIPRegister(
