@@ -19,9 +19,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision$
- * $Author$
- * $Date$
+ * $Revision: 22746 $
+ * $Author: rjongbloed $
+ * $Date: 2009-05-28 01:34:50 +0000 (Thu, 28 May 2009) $
  */
 
 #define _CRT_NONSTDC_NO_DEPRECATE 1
@@ -68,7 +68,7 @@
 
 static void * create_codec(const struct PluginCodec_Definition * codec)
 {
-  int opt = (int)codec->userData;
+  int opt = codec->userData != 0;
   struct gsm_state * context = gsm_create();
   gsm_option(context, GSM_OPT_WAV49, &opt);
   return context;
