@@ -23,9 +23,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 19582 $
+ * $Revision: 21752 $
  * $Author: rjongbloed $
- * $Date: 2008-02-22 07:04:40 +0000 (Fri, 22 Feb 2008) $
+ * $Date: 2008-12-10 02:48:17 +0000 (Wed, 10 Dec 2008) $
  */
 
 #include "stdafx.h"
@@ -38,7 +38,7 @@
 IMPLEMENT_DYNAMIC(COptionsSIP, CDialog)
 
 COptionsSIP::COptionsSIP(CWnd* pParent /*=NULL*/)
-  : CDialog(COptionsSIP::IDD, pParent)
+  : CScrollableDialog(COptionsSIP::IDD, pParent)
   , m_strAddressOfRecord(_T(""))
   , m_strHostName(_T(""))
   , m_strAuthUser(_T(""))
@@ -63,17 +63,13 @@ void COptionsSIP::DoDataExchange(CDataExchange* pDX)
 }
 
 
-BEGIN_MESSAGE_MAP(COptionsSIP, CDialog)
+BEGIN_MESSAGE_MAP(COptionsSIP, CScrollableDialog)
 END_MESSAGE_MAP()
 
 
 BOOL COptionsSIP::OnInitDialog()
 {
-  CDialog::OnInitDialog();
-
-  if (!m_dlgCommandBar.Create(this) || !m_dlgCommandBar.InsertMenuBar(IDR_DIALOGS)) {
-    TRACE0("Failed to create CommandBar\n");
-  }
+  CScrollableDialog::OnInitDialog();
 
   return TRUE;
 }
