@@ -21,9 +21,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 20589 $
+ * $Revision: 21986 $
  * $Author: rjongbloed $
- * $Date: 2008-07-17 03:27:18 +0000 (Thu, 17 Jul 2008) $
+ * $Date: 2009-02-04 04:31:07 +0000 (Wed, 04 Feb 2009) $
  */
 
 #include <ptlib.h>
@@ -365,20 +365,8 @@ void DisplayMediaFormats(const OpalMediaFormatList & mediaList)
     str += fmt->IsValidForProtocol("h.323") ? "yes" : "no";
     str += ", sip=";
     str += fmt->IsValidForProtocol("sip") ? "yes" : "no";
-    cout << setw(max_len+2) << *fmt << "  ";
-    switch (fmt->GetDefaultSessionID()) {
-      case OpalMediaFormat::DefaultAudioSessionID:
-        cout << "audio, bandwidth=" << fmt->GetBandwidth() << ", RTP=" << fmt->GetPayloadType() << str << endl;
-        break;
-      case OpalMediaFormat::DefaultVideoSessionID:
-        cout << "video, bandwidth=" << fmt->GetBandwidth() << ", RTP=" << fmt->GetPayloadType() << str << endl;
-        break;
-      case OpalMediaFormat::DefaultDataSessionID:
-        cout << "data" << endl;;
-        break;
-      default:
-        cout << "unknown type " << hex << fmt->GetDefaultSessionID() << dec << endl;
-    }
+    cout << setw(max_len+2) << *fmt << "  " << fmt->GetMediaType()
+         << ", bandwidth=" << fmt->GetBandwidth() << ", RTP=" << fmt->GetPayloadType() << str << endl;
   }
 
   cout << "\n\n";
@@ -425,15 +413,18 @@ void DisplayMediaFormat(const PString & fmtName)
   }
 
   cout << "        Media Format Name       = " << mediaFormat << "\n"
-          "       Default Session ID (R/O) = " << mediaFormat.GetDefaultSessionID() << "\n"
+          "       Default Session ID (R/O) = " << mediaFormat.GetMediaType().GetDefinition()->GetDefaultSessionId() << "\n"
           "             Payload type (R/O) = " << (unsigned)mediaFormat.GetPayloadType() << "\n"
           "            Encoding name (R/O) = " << mediaFormat.GetEncodingName() << '\n';
   for (PINDEX i = 0; i < mediaFormat.GetOptionCount(); i++) {
     const OpalMediaOption & option = mediaFormat.GetOption(i);
     cout << right << setw(25) << option.GetName() << " (R/" << (option.IsReadOnly() ? 'O' : 'W')
          << ") = " << left << setw(10) << option.AsString();
+#if OPAL_SIP
     if (!option.GetFMTPName().IsEmpty())
       cout << "  FMTP name: " << option.GetFMTPName() << " (" << option.GetFMTPDefault() << ')';
+#endif // OPAL_SIP
+#if OPAL_H323
     const OpalMediaOption::H245GenericInfo & genericInfo = option.GetH245Generic();
     if (genericInfo.mode != OpalMediaOption::H245GenericInfo::None) {
       cout << "  H.245 Ordinal: " << genericInfo.ordinal
@@ -445,6 +436,7 @@ void DisplayMediaFormat(const PString & fmtName)
       if (!genericInfo.excludeReqMode)
         cout << " RM";
     }
+#endif // OPAL_H323
     cout << endl;
   }
   cout << endl;
@@ -454,7 +446,7 @@ void DisplayMediaFormat(const PString & fmtName)
 void AudioTest(const PString & fmtName)
 {
   OpalMediaFormat mediaFormat = fmtName;
-  if (mediaFormat.IsEmpty() || mediaFormat.GetDefaultSessionID() != OpalMediaFormat::DefaultAudioSessionID) {
+  if (mediaFormat.IsEmpty() || mediaFormat.GetMediaType() != OpalMediaType::Audio()) {
     cout << "error: cannot use format name \"" << fmtName << '"' << endl;
     return;
   }
@@ -479,10 +471,8 @@ void AudioTest(const PString & fmtName)
 
   PTones tones("110:0.4-0.1/220:0.4-0.1/440:0.4-0.1/880:0.4-0.1/1760:0.4-0.1");
 
-  RTP_DataFrame source, output, encoded;
-  source.SetPayloadSize(decBlkSize);
-  output.SetPayloadSize(decBlkSize);
-  encoded.SetPayloadSize(PMAX(encoder->GetOptimalDataFrameSize(PFalse), decoder->GetOptimalDataFrameSize(PTrue)));
+  RTP_DataFrame source(decBlkSize), output(decBlkSize);
+  RTP_DataFrame encoded(PMAX(encoder->GetOptimalDataFrameSize(PFalse), decoder->GetOptimalDataFrameSize(PTrue)));
   encoded.SetPayloadType(mediaFormat.GetPayloadType());
 
   int count = 0;
@@ -512,7 +502,7 @@ void AudioTest(const PString & fmtName)
 void VideoTest(const PString & fmtName)
 {
   OpalMediaFormat mediaFormat = fmtName;
-  if (mediaFormat.IsEmpty() || mediaFormat.GetDefaultSessionID() != OpalMediaFormat::DefaultVideoSessionID) {
+  if (mediaFormat.IsEmpty() || mediaFormat.GetMediaType() != OpalMediaType::Video()) {
     cout << "error: cannot use format name \"" << fmtName << '"' << endl;
     return;
   }
@@ -534,8 +524,7 @@ void VideoTest(const PString & fmtName)
   fakeVideo->SetColourFormat(OpalYUV420P);
   fakeVideo->Open("");
 
-  RTP_DataFrame source;
-  source.SetPayloadSize(fakeVideo->GetMaxFrameBytes()+sizeof(OpalVideoTranscoder::FrameHeader));
+  RTP_DataFrame source(fakeVideo->GetMaxFrameBytes()+sizeof(OpalVideoTranscoder::FrameHeader));
 
   RTP_DataFrameList encoded, output;
 
@@ -640,6 +629,7 @@ void OpalCodecInfo::Main()
     needHelp = false;
   }
 
+#if OPAL_H323
   if (args.HasOption('a')) {
     H323CapabilityFactory::KeyList_T keyList = H323CapabilityFactory::GetKeyList();
     cout << "Registered capabilities:" << endl
@@ -660,6 +650,7 @@ void OpalCodecInfo::Main()
     }
     needHelp = false;
   }
+#endif // OPAL_H323
 
   if (args.HasOption('T')) {
     cout << "Available trancoders:" << endl;
