@@ -26,9 +26,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 21846 $
- * $Author: ykiryanov $
- * $Date: 2008-12-19 06:45:12 +0000 (Fri, 19 Dec 2008) $
+ * $Revision: 22067 $
+ * $Author: rjongbloed $
+ * $Date: 2009-02-15 22:59:06 +0000 (Sun, 15 Feb 2009) $
  */
 
 #include <ptlib.h>
@@ -234,7 +234,9 @@ bool H323Gatekeeper::StartGatekeeper(const H323TransportAddress & initialAddress
     return false;
 
   reregisterNow = true;
-  monitorTickle.Signal();
+
+  // Make sure transctor (socket read) thread has started, before kicking off GRQ
+  timeToLive.SetInterval(500);
   return true;
 }
 
@@ -255,14 +257,6 @@ bool H323Gatekeeper::DiscoverGatekeeper()
   
   request.Poll(*this, endpoint.GetGatekeeperRequestRetries(), endpoint.GetGatekeeperRequestTimeout());
   
-  transport->SetInterface(transport->GetLastReceivedInterface());
-  
-  if (discoveryComplete) {
-    PTRACE(3, "RAS\tGatekeeper discovered at: "
-           << transport->GetRemoteAddress()
-           << " (if=" << transport->GetLocalAddress() << ')');
-  }
-  
   requestsMutex.Wait();
   requests.SetAt(request.sequenceNumber, NULL);
   requestsMutex.Signal();
@@ -275,6 +269,8 @@ PBoolean H323Gatekeeper::WriteTo(H323TransactionPDU & pdu,
                                  const H323TransportAddressArray & addresses, 
                                  PBoolean callback)
 {
+  PWaitAndSignal mutex(transport->GetWriteMutex());
+
   if (!discoveryComplete && pdu.GetPDU().GetTag() == H225_RasMessage::e_gatekeeperRequest) {
     if (!transport->WriteConnect(WriteGRQ, &pdu.GetPDU())) {
       PTRACE(1, "RAS\tError writing discovery PDU: " << transport->GetErrorText());
@@ -345,16 +341,21 @@ PBoolean H323Gatekeeper::OnReceiveGatekeeperConfirm(const H225_GatekeeperConfirm
       iterAuth->Enable(iterAuth->IsCapability(gcf.m_authenticationMode, gcf.m_algorithmOID));
   }
 
-  H323TransportAddress locatedAddress(gcf.m_rasAddress, "udp");
-  PTRACE(3, "RAS\tGatekeeper discovered at: "
-         << transport->GetRemoteAddress()
-         << " (if=" << transport->GetLocalAddress() << ')');
+  {
+    PWaitAndSignal mutex(transport->GetWriteMutex());
 
-  if (!transport->SetRemoteAddress(locatedAddress)) {
-    PTRACE(2, "RAS\tInvalid gatekeeper discovery address: \"" << locatedAddress << '"');
-    return PFalse;
+    H323TransportAddress locatedAddress(gcf.m_rasAddress, "udp");
+    if (!transport->SetRemoteAddress(locatedAddress)) {
+      PTRACE(2, "RAS\tInvalid gatekeeper discovery address: \"" << locatedAddress << '"');
+      return PFalse;
+    }
+
+    transport->SetInterface(transport->GetLastReceivedInterface());
+
+    PTRACE(3, "RAS\tGatekeeper discovered at: "
+           << transport->GetRemoteAddress()
+           << " (if=" << transport->GetLocalAddress() << ')');
   }
-  transport->SetInterface(transport->GetLastReceivedInterface());
 
   if (gcf.HasOptionalField(H225_GatekeeperConfirm::e_alternateGatekeeper))
     SetAlternates(gcf.m_alternateGatekeeper, PFalse);
@@ -387,6 +388,38 @@ PBoolean H323Gatekeeper::OnReceiveGatekeeperReject(const H225_GatekeeperReject &
 }
 
 
+bool H323Gatekeeper::SetListenerAddresses(H225_ArrayOf_TransportAddress & pdu)
+{
+  H323TransportAddressArray listeners = endpoint.GetInterfaceAddresses(true, transport);
+  if (listeners.IsEmpty())
+    return false;
+
+  for (PINDEX i = 0; i < listeners.GetSize(); i++) {
+    if (listeners[i].GetProto() != "tcp")
+      continue;
+
+    H225_TransportAddress pduAddr;
+    if (!listeners[i].SetPDU(pduAddr))
+      continue;
+
+    // Check for already have had that address.
+    PINDEX pos, lastPos = pdu.GetSize();
+    for (pos = 0; pos < lastPos; pos++) {
+      if (pdu[pos] == pduAddr)
+        break;
+    }
+
+    if (pos >= lastPos) {
+      // Put new listener into array
+      pdu.SetSize(lastPos+1);
+      pdu[lastPos] = pduAddr;
+    }
+  }
+
+  return pdu.GetSize() > 0;
+}
+
+
 PBoolean H323Gatekeeper::RegistrationRequest(PBoolean autoReg, PBoolean didGkDiscovery)
 {
   if (PAssertNULL(transport) == NULL)
@@ -415,16 +448,11 @@ PBoolean H323Gatekeeper::RegistrationRequest(PBoolean autoReg, PBoolean didGkDis
   
   rasAddress.SetPDU(rrq.m_rasAddress[0]);
 
-  H323TransportAddressArray listeners = endpoint.GetInterfaceAddresses(PTrue, transport);
-  if (listeners.IsEmpty()) {
+  if (!SetListenerAddresses(rrq.m_callSignalAddress)) {
     PTRACE(1, "RAS\tCannot register with Gatekeeper without a H323Listener!");
-    return PFalse;
+    return false;
   }
 
-  for (PINDEX i = 0; i < listeners.GetSize(); i++)
-   if(listeners[i].GetProto() == "tcp")
-    listeners[i].SetPDU(rrq.m_callSignalAddress, *transport);
-
   endpoint.SetEndpointTypeInfo(rrq.m_terminalType);
   endpoint.SetVendorIdentifierInfo(rrq.m_endpointVendor);
 
@@ -683,15 +711,14 @@ void H323Gatekeeper::RegistrationTimeToLive()
   PBoolean didGkDiscovery = false;
 
   if (!discoveryComplete) {
+    timeToLive.SetInterval(0, 0, 1);
     if (endpoint.GetSendGRQ()) {
-      if (DiscoverGatekeeper()) {
-        requiresDiscovery = false;
-        didGkDiscovery = true;
-      } else {
+      if (!DiscoverGatekeeper()) {
         PTRACE_IF(2, !reregisterNow, "RAS\tDiscovery failed, retrying in 1 minute");
-        timeToLive = PTimeInterval(0, 0, 1);
         return;
       }
+      requiresDiscovery = false;
+      didGkDiscovery = true;
     }
     else {
       PTRACE_IF(3, !requiresDiscovery, "RAS\tSkipping gatekeeper discovery for " << transport->GetRemoteAddress());
@@ -732,11 +759,7 @@ PBoolean H323Gatekeeper::UnregistrationRequest(int reason)
 
   H323TransportAddress rasAddress = transport->GetLocalAddress();
 
-  const H323ListenerList & listeners = endpoint.GetListeners();
-  for (H323ListenerList::const_iterator listener = listeners.begin(); listener != listeners.end(); ++listener) {
-    H323TransportAddress signalAddress = listener->GetLocalAddress(rasAddress);
-    signalAddress.SetPDU(urq.m_callSignalAddress, *transport);
-  }
+  SetListenerAddresses(urq.m_callSignalAddress);
 
   urq.IncludeOptionalField(H225_UnregistrationRequest::e_endpointAlias);
   H323SetAliasAddresses(endpoint.GetAliasNames(), urq.m_endpointAlias);
@@ -1450,11 +1473,7 @@ H225_InfoRequestResponse & H323Gatekeeper::BuildInfoRequestResponse(H323RasPDU &
   }
   address.SetPDU(irr.m_rasAddress);
 
-  const H323ListenerList & listeners = endpoint.GetListeners();
-  for (H323ListenerList::const_iterator listener = listeners.begin(); listener != listeners.end(); ++listener) {
-    address = listener->GetLocalAddress();
-    address.SetPDU(irr.m_callSignalAddress, *transport);
-  }
+  SetListenerAddresses(irr.m_callSignalAddress);
 
   irr.IncludeOptionalField(H225_InfoRequestResponse::e_endpointAlias);
   H323SetAliasAddresses(endpoint.GetAliasNames(), irr.m_endpointAlias);
@@ -1635,15 +1654,17 @@ PBoolean H323Gatekeeper::OnReceiveInfoRequest(const H225_InfoRequest & irq)
     return PFalse;
 
   H323TransportAddress oldAddress = transport->GetRemoteAddress();
-  if (!oldAddress.IsEquivalent(replyAddress)) {
-
-    PBoolean ok = transport->ConnectTo(replyAddress) && WritePDU(response);
-    transport->ConnectTo(oldAddress);
-
-    return ok;
-  }
-  else 
+  if (oldAddress.IsEquivalent(replyAddress))
     return WritePDU(response);
+
+  transport->GetWriteMutex().Wait();
+
+  bool ok = transport->ConnectTo(replyAddress) && WritePDU(response);
+  transport->ConnectTo(oldAddress);
+
+  transport->GetWriteMutex().Signal();
+
+  return ok;
 }
 
 
