@@ -26,9 +26,9 @@
  *
  * Contributor(s): Many thanks to Simon Horne.
  *
- * $Revision$
- * $Author$
- * $Date$
+ * $Revision: 23128 $
+ * $Author: rjongbloed $
+ * $Date: 2009-07-22 05:17:45 +0000 (Wed, 22 Jul 2009) $
  */
 
 #include <ptlib.h>
@@ -89,10 +89,6 @@ H323EndPoint::H323EndPoint(OpalManager & manager)
 
   localAliasNames.AppendString(defaultLocalPartyName);
 
-  autoStartReceiveFax = autoStartTransmitFax = PFalse;
-  
-  isH224Enabled = PFalse;
-
   m_bH245Disabled = PFalse;
   autoCallForward = PTrue;
   disableFastStart = PFalse;
@@ -160,25 +156,32 @@ void H323EndPoint::SetEndpointTypeInfo(H225_EndpointType & info) const
     case e_TerminalAndMC :
       info.IncludeOptionalField(H225_EndpointType::e_terminal);
       break;
+
     case e_GatewayOnly :
     case e_GatewayAndMC :
     case e_GatewayAndMCWithDataMP :
     case e_GatewayAndMCWithAudioMP :
     case e_GatewayAndMCWithAVMP :
       info.IncludeOptionalField(H225_EndpointType::e_gateway);
+      if (SetGatewaySupportedProtocol(info.m_gateway.m_protocol))
+        info.m_gateway.IncludeOptionalField(H225_GatewayInfo::e_protocol);
       break;
+
     case e_GatekeeperOnly :
     case e_GatekeeperWithDataMP :
     case e_GatekeeperWithAudioMP :
     case e_GatekeeperWithAVMP :
       info.IncludeOptionalField(H225_EndpointType::e_gatekeeper);
       break;
+
     case e_MCUOnly :
     case e_MCUWithDataMP :
     case e_MCUWithAudioMP :
     case e_MCUWithAVMP :
       info.IncludeOptionalField(H225_EndpointType::e_mcu);
-      info.m_mc = PTrue;
+      info.m_mc = true;
+      if (SetGatewaySupportedProtocol(info.m_mcu.m_protocol))
+        info.m_mcu.IncludeOptionalField(H225_McuInfo::e_protocol);
   }
 }
 
@@ -205,6 +208,36 @@ void H323EndPoint::SetH221NonStandardInfo(H225_H221NonStandard & info) const
 }
 
 
+bool H323EndPoint::SetGatewaySupportedProtocol(H225_ArrayOf_SupportedProtocols & protocols) const
+{
+  PStringList prefixes;
+
+  if (!OnSetGatewayPrefixes(prefixes))
+    return false;
+
+  PINDEX count = protocols.GetSize();
+  protocols.SetSize(count+1);
+
+  protocols[count].SetTag(H225_SupportedProtocols::e_h323);
+  H225_H323Caps & caps = protocols[count];
+
+  caps.IncludeOptionalField(H225_H323Caps::e_supportedPrefixes);
+  H225_ArrayOf_SupportedPrefix & supportedPrefixes = caps.m_supportedPrefixes;	
+  supportedPrefixes.SetSize(prefixes.GetSize());
+
+  for (PINDEX i = 0; i < prefixes.GetSize(); i++)
+    H323SetAliasAddress(prefixes[i], supportedPrefixes[i].m_prefix);
+
+  return true;
+}
+
+
+bool H323EndPoint::OnSetGatewayPrefixes(PStringList & prefixes) const
+{
+	return false;
+}
+
+
 H323Capability * H323EndPoint::FindCapability(const H245_Capability & cap) const
 {
   return GetCapabilities().FindCapability(cap);
@@ -283,7 +316,7 @@ PBoolean H323EndPoint::UseGatekeeper(const PString & address,
   if (gatekeeper != NULL) {
     PBoolean same = PTrue;
 
-    if (!address)
+    if (!address && address != "*")
       same = gatekeeper->GetTransport().GetRemoteAddress().IsEquivalent(address);
 
     if (!same && !identifier)
@@ -307,7 +340,7 @@ PBoolean H323EndPoint::UseGatekeeper(const PString & address,
       transport = new H323TransportUDP(*this, ip, port);
   }
 
-  if (address.IsEmpty()) {
+  if (address.IsEmpty() || address == "*") {
     if (identifier.IsEmpty())
       return DiscoverGatekeeper(transport);
     else
@@ -603,14 +636,18 @@ PBoolean H323EndPoint::InternalMakeCall(OpalCall & call,
     return PFalse;
   }
 
-  // Restriction: the call must be made on the same transport as the one
+  // Restriction: the call must be made on the same local interface as the one
   // that the gatekeeper is using.
   H323Transport * transport;
   if (gatekeeper != NULL)
     transport = gatekeeper->GetTransport().GetLocalAddress().CreateTransport(
                                           *this, OpalTransportAddress::Streamed);
-  else
+  else if (stringOptions == NULL || !stringOptions->Contains(OPAL_OPT_INTERFACE))
     transport = address.CreateTransport(*this, OpalTransportAddress::NoBinding);
+  else {
+    OpalTransportAddress localInterface = (*stringOptions)[OPAL_OPT_INTERFACE];
+    transport = localInterface.CreateTransport(*this, OpalTransportAddress::HostOnly);
+  }
 
   if (transport == NULL) {
     PTRACE(1, "H323\tInvalid transport in \"" << remoteParty << '"');
@@ -755,8 +792,8 @@ PBoolean H323EndPoint::ParsePartyName(const PString & remoteParty,
         PIPSocketAddressAndPortVector addresses;
         if (PDNS::LookupSRV(hostname, "_h323._tcp", url.GetPort(), addresses) && !addresses.empty()) {
           // Only use first entry
-          url.SetHostName(addresses[0].address.AsString());
-          url.SetPort(addresses[0].port);
+          url.SetHostName(addresses[0].GetAddress().AsString());
+          url.SetPort(addresses[0].GetPort());
         }
       }
     }
