@@ -24,9 +24,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 22647 $
+ * $Revision: 23294 $
  * $Author: rjongbloed $
- * $Date: 2009-05-19 04:14:23 +0000 (Tue, 19 May 2009) $
+ * $Date: 2009-08-28 06:26:26 +0000 (Fri, 28 Aug 2009) $
  */
 
 #include <ptlib.h>
@@ -274,14 +274,14 @@ void OpalManager::AttachEndPoint(OpalEndPoint * endpoint, const PString & prefix
   PWriteWaitAndSignal mutex(endpointsMutex);
 
   if (endpointMap.find(thePrefix) != endpointMap.end()) {
-    PTRACE(1, "OpalMan\tCannot re-register endpoint prefix " << thePrefix);
+    PTRACE(1, "OpalMan\tCannot re-attach endpoint prefix " << thePrefix);
     return;
   }
 
   if (endpointList.GetObjectsIndex(endpoint) == P_MAX_INDEX)
     endpointList.Append(endpoint);
   endpointMap[thePrefix] = endpoint;
-  PTRACE(1, "OpalMan\tRegistered endpoint with prefix " << thePrefix);
+  PTRACE(3, "OpalMan\tAttached endpoint with prefix " << thePrefix);
 }
 
 
