@@ -23,9 +23,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 22995 $
- * $Author: rjongbloed $
- * $Date: 2009-06-26 06:23:20 +0000 (Fri, 26 Jun 2009) $
+ * $Revision: 23347 $
+ * $Author: dsandras $
+ * $Date: 2009-09-05 15:16:49 +0000 (Sat, 05 Sep 2009) $
  */
 
 #include <ptlib.h>
@@ -76,6 +76,7 @@ SIPEndPoint::SIPEndPoint(OpalManager & mgr)
 #if OPAL_HAS_SIPIM
   , m_sipIMManager(*this)
 #endif
+  , m_disableTrying(true)
 
 #ifdef _MSC_VER
 #pragma warning(default:4355)
@@ -158,29 +159,62 @@ PString SIPEndPoint::GetDefaultTransport() const
 
 PBoolean SIPEndPoint::NewIncomingConnection(OpalTransport * transport)
 {
-  PTRACE_IF(2, transport->IsReliable(), "SIP\tListening thread started.");
-
   transport->SetBufferSize(SIP_PDU::MaxSize);
 
-  do {
-    HandlePDU(*transport);
-  } while (transport->IsOpen() && transport->IsReliable() && !transport->bad() && !transport->eof());
+  if (transport->IsReliable()) {
+    // TCP connection, read until closed
+    TransportReadLoop(transport);
+    return false; // Do not delete the transport!
+  }
 
-  PTRACE_IF(2, transport->IsReliable(), "SIP\tListening thread finished.");
-
-  return PTrue;
+  // UDP packet, handle it and return
+  HandlePDU(*transport);
+  return true;
 }
 
 
 void SIPEndPoint::TransportThreadMain(PThread &, INT param)
+{
+  TransportReadLoop((OpalTransport *)param);
+}
+
+
+void SIPEndPoint::TransportReadLoop(OpalTransport * transport)
 {
   PTRACE(4, "SIP\tRead thread started.");
-  OpalTransport * transport = (OpalTransport *)param;
 
   do {
     HandlePDU(*transport);
   } while (transport->IsOpen() && !transport->bad() && !transport->eof());
-  
+
+  /* This is increadibly ugly, but a true fix requires quite a substantial rewrite.
+
+     The problem is that a TCP connection is SIP could be associated with multiple
+     SIP connection objects, that is INVITEs to different calls could come down
+     the same TCP connection. At present there is a pointer to the TCP transport
+     in each SIPConnection instance and we have to wait for all of this connections
+     to finish with the pointer before we delete the transport.
+
+     This clearly requires a reference counting system, or better yet removal of the
+     pointer to the transport completely so all writes of PDU's to a remote are done
+     via a function on the endpoint which automatically selects the appropriate
+     transport from a poll it has.
+   */
+
+  PTRACE(4, "SIP\tRead transport closed, checking connections that use it.");
+  PSafePtr<SIPConnection> connection = PSafePtrCast<OpalConnection, SIPConnection>(connectionsActive.GetAt(0, PSafeReference));
+  while (connection != NULL) {
+    if (&connection->GetTransport() == transport) {
+      connection->Release(OpalConnection::EndedByTransportFail);
+      while (&connection->GetTransport() == transport)
+        PThread::Sleep(100);
+    }
+
+    ++connection;
+  }
+
+  delete connection;
+
   PTRACE(4, "SIP\tRead thread finished.");
 }
 
@@ -252,12 +286,9 @@ OpalTransport * SIPEndPoint::CreateTransport(const SIPURL & remoteURL, const PSt
   }
 
   if (transport == NULL) {
-    // No compatible listeners, can't use their binding
-    transport = remoteAddress.CreateTransport(*this, OpalTransportAddress::NoBinding);
-    if (transport == NULL) {
-      PTRACE(1, "SIP\tCould not create transport for " << remoteAddress);
+    // No compatible listeners, can't create a transport to send if we cannot hear the responses!
+    PTRACE(2, "SIP\tNo compatible listener to create transport for " << remoteAddress);
       return NULL;
-    }
   }
 
   if (!transport->SetRemoteAddress(remoteAddress)) {
@@ -510,7 +541,8 @@ PBoolean SIPEndPoint::OnReceivedPDU(OpalTransport & transport, SIP_PDU * pdu)
       // Do next case
 
     default :
-      pdu->SendResponse(transport, SIP_PDU::Information_Trying, this);
+      if (!m_disableTrying || (pdu->GetMethod() == SIP_PDU::Method_INVITE))
+        pdu->SendResponse(transport, SIP_PDU::Information_Trying, this);
       // Do next case
 
     case SIP_PDU::Method_ACK :
@@ -657,39 +689,11 @@ PBoolean SIPEndPoint::OnReceivedSUBSCRIBE(OpalTransport & transport, SIP_PDU & p
 
 void SIPEndPoint::OnReceivedResponse(SIPTransaction & transaction, SIP_PDU & response)
 {
-  switch (response.GetStatusCode()) {
-    case SIP_PDU::Failure_IntervalTooBrief :
-      OnReceivedIntervalTooBrief(transaction, response);
-      break;
-
-    case SIP_PDU::Failure_UnAuthorised :
-    case SIP_PDU::Failure_ProxyAuthenticationRequired :
-      OnReceivedAuthenticationRequired(transaction, response);
-      break;
-
-    case SIP_PDU::Failure_RequestTimeout :
-      OnTransactionFailed(transaction);
-      break;
-
-    default :
-      switch (response.GetStatusCode()/100) {
-        case 1 :
-          // Do nothing on 1xx
-          break;
-
-        case 2 :
-          OnReceivedOK(transaction, response);
-          break;
-
-        default :
-          PSafePtr<SIPHandler> handler = activeSIPHandlers.FindSIPHandlerByCallID(response.GetMIME().GetCallID(), PSafeReadWrite);
-          if (handler != NULL) 
-            handler->OnFailed(response.GetStatusCode());
-          else {
-            PTRACE(2, "SIP\tResponse received for unknown handler ID: " << response.GetMIME().GetCallID());
-          }
-          break;
-      }
+  PSafePtr<SIPHandler> handler = activeSIPHandlers.FindSIPHandlerByCallID(response.GetMIME().GetCallID(), PSafeReadWrite);
+  if (handler != NULL)
+    handler->OnReceivedResponse(transaction, response);
+  else {
+    PTRACE(2, "SIP\tResponse received for unknown handler ID: " << response.GetMIME().GetCallID());
   }
 }
 
@@ -836,50 +840,6 @@ PBoolean SIPEndPoint::OnReceivedINVITE(OpalTransport & transport, SIP_PDU * requ
   return PTrue;
 }
 
-void SIPEndPoint::OnReceivedIntervalTooBrief(SIPTransaction & transaction, SIP_PDU & response)
-{
-  const SIPMIMEInfo & responseMIME = response.GetMIME();
-  PSafePtr<SIPHandler> handler = activeSIPHandlers.FindSIPHandlerByCallID(responseMIME.GetCallID(), PSafeReadWrite);
-  if (handler == NULL) {
-    PTRACE(2, "SIP\tInterval too brief received for unknown handler ID: " << response.GetMIME().GetCallID());
-    return;
-  }
-
-  SIPTransaction *newTransaction = handler->CreateTransaction(transaction.GetTransport());
-  if (newTransaction) {
-    handler->SetExpire(responseMIME.GetMinExpires());
-    newTransaction->GetMIME().SetExpires(responseMIME.GetMinExpires());
-    newTransaction->GetMIME().SetCallID(handler->GetCallID());
-    if (newTransaction->Start())
-      return;
-  }
-
-  PTRACE(1, "SIP\t Could not restart REGISTER after IntervalTooBrief error!");
-  handler->OnFailed(SIP_PDU::Failure_IntervalTooBrief);
-}
-
-void SIPEndPoint::OnReceivedAuthenticationRequired(SIPTransaction & transaction, SIP_PDU & response)
-{
-  // Try to find authentication information for the given call ID
-  PSafePtr<SIPHandler> handler = activeSIPHandlers.FindSIPHandlerByCallID(response.GetMIME().GetCallID(), PSafeReadWrite);
-  if (handler != NULL)
-    handler->OnReceivedAuthenticationRequired(transaction, response);
-  else {
-    PTRACE(2, "SIP\tAuthentication required received for unknown handler ID: " << response.GetMIME().GetCallID());
-  }
-}
-
-
-void SIPEndPoint::OnReceivedOK(SIPTransaction & transaction, SIP_PDU & response)
-{
-  PSafePtr<SIPHandler> handler = activeSIPHandlers.FindSIPHandlerByCallID(response.GetMIME().GetCallID(), PSafeReadWrite);
-  if (handler != NULL) 
-    handler->OnReceivedOK(transaction, response);
-  else {
-    PTRACE(2, "SIP\tOK received for unknown handler ID: " << response.GetMIME().GetCallID());
-  }
-}
-    
 
 void SIPEndPoint::OnTransactionFailed(SIPTransaction & transaction)
 {
@@ -1075,11 +1035,7 @@ bool SIPEndPoint::Register(const SIPRegister::Params & params, PString & aor)
 
   aor = handler->GetAddressOfRecord().AsString();
 
-  if (handler->ActivateState(SIPHandler::Subscribing))
-    return true;
-
-  activeSIPHandlers.Remove(handler);
-  return false;
+  return handler->ActivateState(SIPHandler::Subscribing);
 }
 
 
@@ -1398,7 +1354,7 @@ unsigned SIPEndPoint::GetAllowedMethods() const
 PBoolean SIPEndPoint::GetAuthentication(const PString & authRealm, PString & realm, PString & user, PString & password) 
 {
   PSafePtr<SIPHandler> handler = activeSIPHandlers.FindSIPHandlerByAuthRealm(authRealm, user, PSafeReadOnly);
-  if (handler == NULL)
+  if (handler == NULL || handler->GetPassword ().IsEmpty ())
     return PFalse;
 
   realm    = handler->GetRealm();
