@@ -23,9 +23,9 @@
  *
  * Contributor(s): Vyacheslav Frolov.
  *
- * $Revision: 21660 $
+ * $Revision: 22029 $
  * $Author: csoutheren $
- * $Date: 2008-12-01 08:21:58 +0000 (Mon, 01 Dec 2008) $
+ * $Date: 2009-02-10 05:25:00 +0000 (Tue, 10 Feb 2009) $
  */
 
 /*
@@ -92,7 +92,7 @@ PString SDPFaxMediaDescription::GetSDPMediaType() const
 
 SDPMediaFormat * SDPFaxMediaDescription::CreateSDPMediaFormat(const PString & portString)
 {
-  return new SDPMediaFormat(RTP_DataFrame::DynamicBase, portString);
+  return new SDPMediaFormat(*this, RTP_DataFrame::DynamicBase, portString);
 }
 
 
